/**********************************************************************/
/*! \class RtMidi
    \brief An abstract base class for realtime MIDI input/output.

    This class implements some common functionality for the realtime
    MIDI input/output subclasses RtMidiIn and RtMidiOut.

    RtMidi WWW site: http://music.mcgill.ca/~gary/rtmidi/

    RtMidi: realtime MIDI i/o C++ classes
    Copyright (c) 2003-2010 Gary P. Scavone

		Modified by S Jagannathan for use in din
		July 2010.
*/
/**********************************************************************/

// RtMidi: Version 1.0.11

#ifndef RTMIDI_H
#define RTMIDI_H

#include "RtError.h"
#include <alsa/asoundlib.h>
#include <string>

// A structure to hold variables related to the ALSA API
// implementation.
struct AlsaMidiData {
  snd_seq_t *seq;
  int vport;
  snd_seq_port_subscribe_t *subscription;
  snd_midi_event_t *coder;
  int bufferSize;
  unsigned char buffer[256];
  pthread_t thread;
};

class RtMidi {
 
 public:

  virtual int openPort( unsigned int portNumber = 0, const std::string portName = std::string( "RtMidi" ) ) = 0;
  virtual unsigned int getPortCount() = 0;
  virtual std::string getPortName( unsigned int portNumber = 0 ) = 0;
  virtual void closePort( void ) = 0;

  RtMidi();
  virtual ~RtMidi() {};

  AlsaMidiData apiData;

  bool connected;
  int portnum;
  std::string portname;


  void error( RtError::Type type );
  std::string errorString;

};

#include <vector>
#include <queue>

class RtMidiIn : public RtMidi
{

 public:

  RtMidiIn( const std::string clientName = std::string( "RtMidi Input Client") );
  ~RtMidiIn();
  
  unsigned int getPortCount();
  int openPort( unsigned int portNumber = 0, const std::string Portname = std::string( "RtMidi Input" ) );
  void closePort();
	std::string getPortName( unsigned int portNumber = 0 );

  
  void setQueueSizeLimit( unsigned int queueSize );
  void getMessage( std::vector<unsigned char> *message );
    
  struct RtMidiInData {
        
    std::vector<unsigned char> message;
    std::queue< std::vector<unsigned char> > queue;
    unsigned int queueLimit;
    bool doInput;
    AlsaMidiData *apiData;
    void *userData;
    bool continueSysex;
    RtMidiInData () : queueLimit(1024), doInput(false), apiData(0), userData(0), continueSysex(false) {}
  };

 private:

  void initialize( const std::string& clientName );
  RtMidiInData inputData;

};


#endif
