#ifndef __AUDIO
#define __AUDIO

#include <jack/jack.h>
#include "console.h"
#include <iostream>

typedef jack_default_audio_sample_t sample_t; // this is just float in jack/types.h

struct audio_out {

	static const int num_channels = 2;
	int samples_per_channel;
	int samples_per_buffer; // has 2 channels
	int samples_buffer_size;
	sample_t sample_rate;

	// written by the main thread
	//
	// a multi buffer scheme
	//
	// main thread writes a buffer, while audio thread reads another
	//
	float* samples_buffers; // a bunch of sample buffers
	int num_samples_buffers;
	float *readp, *writep;
	int* available; // buffer? available for audio out?
	int readi, writei;

	// buffers for result, amplitude modulation & fm modulation
	//
	float *result, *ams, *fms;

	//
	// jack
	//

	jack_client_t *client;

	// L and R connections
	int lcons, rcons; // l and r connected?
	jack_port_t *out_left, *out_right;
	const char **con_left, **con_right;

	std::string prefs_name;
	void load_prefs ();
	void save_prefs ();
	void defaults ();

	audio_out ();
	~audio_out ();

	int open ();
	int close ();
	int start ();
	int stop ();
	void write (float* abuf, float* lbuf, float* rbuf);
	void alloc ();
	void set_sample_rate (int s);

};

int audio_wanted (jack_nframes_t nsamples, void *arg);
int buffer_size_changed (jack_nframes_t nframes, void *arg);
int sample_rate_changed (jack_nframes_t nframes, void *arg);

console& operator<< (console& c, const audio_out& ao);

#endif
