/*
 * din irc bot
 *
 * based on LGPLed code by Georgy Yunaev gyunaev@ulduzsoft.com
 * of libircclient.sourceforge.net.
 *
*/

#ifndef __BOT
#define __BOT

#include <libircclient/libircclient.h>
#include <cstring>
#include "command.h"
#include "globals.h"

void* irc_monitor (void *pbot);

// callbacks for various irc events
void dump_event (irc_session_t* session, const char* event, const char* origin, const char** params, unsigned int count);
void event_join (irc_session_t* session, const char* event, const char* origin, const char** params, unsigned int count);
void event_connect (irc_session_t* session, const char* event, const char* origin, const char** params, unsigned int count);
void event_privmsg (irc_session_t* session, const char* event, const char* origin, const char** params, unsigned int count);
void event_channel (irc_session_t* session, const char* event, const char* origin, const char** params, unsigned int count);
void irc_event_dcc_chat (irc_session_t* session, const char* nick, const char* addr, irc_dcc_t dccid);
void irc_event_dcc_send (irc_session_t* session, const char* nick, const char* addr, const char* filename, unsigned long size, irc_dcc_t dccid);
void event_numeric (irc_session_t* session, unsigned int event, const char* origin, const char** params, unsigned int count);

// utils
std::string get_nick (const char * origin);
struct tokenizer;

// the bot
//
struct bot : command, pusher<bot> {

	// libircclient
	irc_callbacks_t	callbacks;
	irc_session_t* sess;

	// server, nickname and channel
	std::string srv, nick, chan;

	std::string nickolon; // nick:
	int len;

	// messages from irc
	std::string mesg;
	std::queue<std::string> cmds, mesgs;
	void handle_input ();

	// irc monitoring thread
	pthread_t tid;

	int prefixed; // yes?
	std::string prefix; // eg., some_nick:

	bot (const std::string& ln, const std::string& sn);
	int make ();

	~bot ();

	std::string quitmsg;
	int kill ();

	bool operator() (tokenizer& ss); // command parser

	// pusher
	bot& operator<< (const std::string& s);
	bot& operator<< (char c) {return *this;}


};

#endif
