#ifndef __command
#define __command

#include <vector>
#include <string>
#include "help.h"
#include "RtMidi.h"

struct tokenizer;

struct command {

	std::string longname;
	std::string shortname;

	command (const std::string& ln, const std::string& sn) : longname (ln), shortname (sn) {}
	virtual bool operator() (tokenizer& tz) { return true; }
	virtual ~command () {}

};

struct cmdlist {

	std::vector<command*> cmds;
	int ncmds;

	std::string result; // holds last run command output

	static const int size = 2048;
	char buf [size]; // for console printing

	void add (command* cmd);

};

struct din;

struct setv : command { // set var
	din* d;
	public:
		setv (din* dd, const std::string& ln, const std::string& sn) : command (ln, sn), d(dd) {}
		bool operator() (tokenizer& tz);
		~setv () {}
};

struct getv : command { // get var
	din* d;
	public:
		getv (din* dd, const std::string& ln, const std::string& sn) : command (ln, sn), d(dd) {}
		bool operator() (tokenizer& tz);
		~getv () {}
};

struct delay;
struct set_delay: command {
	delay* left;
	delay* right;
	public:
		set_delay (delay* l, delay* r, const std::string& ln, const std::string& sn) : command (ln, sn), left(l), right(r) {}
		bool operator() (tokenizer& tz);
		~set_delay () {}
};

struct get_delay: command {
	delay* left;
	delay* right;
	public:
		get_delay (delay* l, delay* r, const std::string& ln, const std::string& sn) : command (ln, sn), left(l), right(r) {}
		bool operator() (tokenizer& tz);
		~get_delay () {}
};

struct beat2value;

// bpm
//
//
struct bpm_com {
	static const int NUM = 5;
	static const char* str [NUM];
	static beat2value* bv [NUM];
	int getid (const std::string& name);
};

struct ls_bpm: command, bpm_com {
	ls_bpm (const std::string& ln, const std::string& sn) : command (ln, sn) {}
	bool operator() (tokenizer& tz);
	~ls_bpm () {}
};

struct set_bpm: command, bpm_com {
	set_bpm (const std::string& ln, const std::string& sn) : command (ln, sn) {}
	bool operator() (tokenizer& tz);
	~set_bpm () {}
};

struct get_bpm: command, bpm_com {
	get_bpm (const std::string& ln, const std::string& sn) : command (ln, sn) {}
	bool operator() (tokenizer& tz);
	~get_bpm () {}
};

struct set_style : command, bpm_com {
	set_style (const std::string& ln, const std::string& sn) : command (ln, sn) {}
	bool operator() (tokenizer& tz);
	~set_style () {}
};

struct get_style : command, bpm_com {
	get_style (const std::string& ln, const std::string& sn) : command (ln, sn) {}
	bool operator() (tokenizer& tz);
	~get_style () {}
};

struct gaters : command {
	din* d;
	gaters (din* dd, const std::string& ln, const std::string& sn) : command (ln, sn), d(dd) {}
	bool operator() (tokenizer& tz);
	void set_state (const std::string& target, const bool& state);
	std::string get_state (const std::string& target);
	~gaters () {}
};

struct add_scale: command {
	public:
		add_scale (const std::string& ln, const std::string& sn) : command (ln, sn) {}
		bool operator () (tokenizer& tz);
		~add_scale () {}
};

struct remove_scale : command {
	public:
		remove_scale (const std::string& ln, const std::string& sn) : command (ln, sn) {}
		bool operator () (tokenizer& tz);
		~remove_scale () {}
};

struct ls_scale: command {
	public:
		ls_scale (const std::string& ln, const std::string& sn) : command (ln, sn) {}
		bool operator () (tokenizer& tz);
		~ls_scale () {}
};

struct ls_notes: command {
	public:
		ls_notes (const std::string& ln, const std::string& sn) : command (ln, sn) {}
		bool operator () (tokenizer& tz);
		~ls_notes () {}
};

struct load_scale : command {
	din* d;
	public:
		load_scale (din* dd, const std::string& ln, const std::string& sn) : command (ln, sn), d(dd) {}
		bool operator () (tokenizer& tz);
		~load_scale () {}
};

struct key : command {
	din* d;
	public:
		key (din* dd, const std::string& ln, const std::string& sn) : command (ln, sn), d(dd) {}
		bool operator() (tokenizer& tz);
		~key () {}
		void find_nearest_note (std::string& note, float& frequency, float& dist);
		bool check (float f);
};

struct load_tuning: command {
	din* d;
	public:
		load_tuning (din* dd, const std::string& ln, const std::string& sn) : command (ln, sn), d(dd) {}
		bool operator() (tokenizer& tz);
		~load_tuning () {}
};

struct notation : command {

	din* d;
	notation (din* dd, const std::string& ln, const std::string& sn) : command (ln, sn), d(dd) {}
	bool operator() (tokenizer& tz);
	~notation () {}
};


struct note_distance: command {
	public:
		note_distance (const std::string& ln, const std::string& sn) : command (ln, sn) {}
		bool operator() (tokenizer& tz);
		~note_distance () {}
};

struct chord: command {
	public:
		chord (const std::string& ln, const std::string& sn) : command (ln, sn) {}
		bool operator() (tokenizer& tz);
		~chord () {}
};

struct set_font_size: command {

	public:
		set_font_size (const std::string& ln, const std::string& sn) : command (ln, sn) {}
		bool operator() (tokenizer& tz);
		~set_font_size () {}

};

struct get_font_size: command {

	public:
		get_font_size (const std::string& ln, const std::string& sn) : command (ln, sn) {}
		bool operator() (tokenizer& tz);
		~get_font_size () {}

};

struct set_kern: command {

	public:
		set_kern (const std::string& ln, const std::string& sn) : command (ln, sn) {}
		bool operator() (tokenizer& tz);
		~set_kern () {}

};

struct get_kern: command {

	public:
		get_kern (const std::string& ln, const std::string& sn) : command (ln, sn) {}
		bool operator() (tokenizer& tz);
		~get_kern () {}

};

struct load_resonators : command {
	din* d;
	load_resonators (din* dd, const std::string& ln, const std::string& sn) : command (ln, sn), d(dd) {}
	bool operator() (tokenizer& tz);
	~load_resonators () {}
};

struct save_resonators : command {
	din* d;
	save_resonators (din* dd, const std::string& ln, const std::string& sn) : command (ln, sn), d(dd) {}
	bool operator() (tokenizer& tz);
	~save_resonators () {}
};

struct delete_disabled_resonators : command {
	din* d;
	delete_disabled_resonators (din* dd, const std::string& ln, const std::string& sn) : command (ln, sn), d(dd) {}
	bool operator() (tokenizer& tz);
	~delete_disabled_resonators () {}
};

struct curve_name : command { // change picked curve name
	curve_name (const std::string& ln, const std::string& sn) : command (ln, sn) {}
	bool operator() (tokenizer& tz);
	~curve_name () {}
};

struct multi_curve;

struct paste_gaters: command {
	paste_gaters (const std::string& ln, const std::string& sn) : command (ln, sn) {}
	bool operator() (tokenizer& tz);
	~paste_gaters () {}
};

struct curve_value : command { // change curve value (vertex, tangent)

	curve_value (const std::string& ln, const std::string& sn) : command (ln, sn) {}
	~curve_value () {}
	bool operator() (tokenizer& tz);

};

struct curve__library : command { // edit curve library attached to current curve editor
	curve__library (const std::string& ln, const std::string& sn) : command (ln, sn) {}
	bool operator() (tokenizer& tz);
	~curve__library () {}
};

struct set_curve_editor : command {
	set_curve_editor (const std::string& ln, const std::string& sn) : command (ln, sn) {}
	bool operator() (tokenizer& tz);
	~set_curve_editor () {}
};

struct get_curve_editor : command {
	get_curve_editor (const std::string& ln, const std::string& sn) : command (ln, sn) {}
	bool operator() (tokenizer& tz);
	~get_curve_editor () {}
};

struct midi : command {

	static const int MAX_DEVICES = 8;
	RtMidiIn devices [MAX_DEVICES];
	int nports;

	midi (const std::string& ln, const std::string& sn) : command (ln, sn) { nports = 0; }
	bool operator() (tokenizer& tz);
	void list_devices ();
	int find_free_device ();
	int open_port_on_device (int port, int dev);
	int port_opened (int port);
	void close_port (int port);
	void handle_input ();
	void run (const std::string& cmd, const std::vector<unsigned char>& msg);
	~midi () {}
};


struct set_scope : command {
	din* d;
	set_scope (const std::string& ln, const std::string& sn, din* dd) : command (ln, sn), d(dd) {}
	bool operator() (tokenizer& tz);
	~set_scope () {}
};

struct get_scope : command {
	din* d;
	get_scope (const std::string& ln, const std::string& sn, din* dd) : command (ln, sn), d(dd) {}
	bool operator() (tokenizer& tz);
	~get_scope () {}
};


struct set_drone : command {
	din& d;
	set_drone (const std::string& ln, const std::string& sn, din& dd) : command (ln, sn), d(dd) {}
	bool operator() (tokenizer& tz);
	~set_drone () {}
};

struct get_drone : command {
	din& d;
	get_drone (const std::string& ln, const std::string& sn, din& dd) : command (ln, sn), d(dd) {}
	bool operator() (tokenizer& tz);
	~get_drone () {}
};

struct echo : command {
	echo (const std::string& ln, const std::string& sn) : command (ln, sn) {}
	bool operator() (tokenizer& tz);
	~echo () {}
};

struct set_text_color: command {
	set_text_color (const std::string& ln, const std::string& sn) : command (ln, sn) {}
	bool operator() (tokenizer& tz);
	~set_text_color () {}
};

struct set_kb_layout : command {
	std::string layout;
	set_kb_layout (const std::string& ln, const std::string& sn) : command (ln, sn), layout ("us") {}
	bool operator() (tokenizer& tz);
	~set_kb_layout () {}
};

struct audio__out: command {
	din& d;
	audio__out (din& dd, const std::string& ln, const std::string& sn) : command (ln, sn), d(dd) {}
	bool operator () (tokenizer& tz);
	void update ();
	~audio__out () {}
};


#endif
