#ifndef _CONSOLE
#define _CONSOLE

#define eol console::EOL
#define cll console::CLL
#define tab "  "

#include <SDL/SDL_opengl.h>
#include "box.h"
#include "ui.h"
#include "globals.h"
#include <vector>
#include <string>
#include <sstream>



struct decoration {
  float r, g, b; // color
  int lw; // line width
  decoration (float rr = 0.9, float gg = 0.9, float bb = 0.9, int w = 1);
};

struct mesg {
  std::string text; // text
  decoration dec; // decoration
  mesg (const std::string& t, const decoration& d) : text(t), dec(d) {}
  mesg (const std::string& t) : text(t) {}
  mesg () {}
};

struct console : ui, pusher<console> {

	static std::string charset [];

	static const char EOL = '\n'; // end of last mesg
	static const char CLL = '\b'; // clear last mesg

  std::vector<mesg> lines;
  int nlines;
  int lines_per_screen;
  int startl;

	mesg scratch_line;
	decoration dec;

	console ();
	console& operator() (const std::string& cmd);
	console& operator<< (unsigned int i);
	console& operator<< (unsigned long i);
	console& operator<< (unsigned long long i);
	console& operator<< (int i);
	console& operator<< (const std::string& s);
	console& operator<< (float f);
	console& operator<< (double d);
	console& operator<< (char c);
	console& operator<< (const decoration& d);
  void push_back (const mesg& ln);

  void up (int i);
  void down (int i);
  void last (); // ensures last mesg is always displayed
	void del ();
	void clear ();
	void pgdn ();
	void pgup ();
	void home ();
	void end ();

  box<int> win;
	void set_window (const box<int>& w);

	static const decoration light;
	static const decoration yellow;
	static const decoration green;
	static const decoration red;
	static const decoration cyan;
	static const decoration bold;
  int startx, starty;
  int char_width, line_height;
	void calc_draw_params ();
	void draw ();

	bool handle_input ();

	// command line
	//
  mesg cmd_line;
	std::vector<std::string> history; // command history
	int hid;
	int command_mode;
	int curs_loc;
	int curs_locx;

	int rollup;

	std::string ps1;

};

template <class T> inline console& operator<< (console& c, const box<T>& b) {
	c << b.left << ' ' << b.bottom << ' ' << b.right << ' ' << b.top;
	return c;
}

extern console cons;
#endif
