#ifndef __font_editor
#define __font_editor

#include "viewwin.h"
#include "ui.h"
#include "point.h"
#include "line.h"
#include "help.h"
#include "curve_editor.h"
#include "glyph.h"
#include "font.h"
#include <map>
using namespace std;

struct line_hit {
  int l; // line?
  int v; // vertex?
  line_hit () {
    l = v = -1; // invalid
  }
  line_hit (int ll, int vv) : l(ll), v(vv) {}
  bool operator() () {
    return ( (l > -1) && (v > -1));
  }
};

struct font_editor : curve_editor {

  font& fn;

  int nchars;
  string name;

  map<char, glyph> glyphs;
  vector<char> chars;
  glyph gl;
  vector<line> lines;
  int ichar;

  vector< point<float> > scratch_points;

  line_hit cur, ins, del;
  bool moving;

  void prep_edit ();
  void prep_save ();
  void edit_char (int i);
  void save_char (int i);

	font_editor (font& f, const string& settingsf, const string& helpf = "~/.din/font_editor.help");
	virtual ~font_editor ();

	void save_font ();


	bool handle_input ();
	void draw ();
	void draw_lines ();
	void draw_scratch_line ();

	line_hit hittest ();

	void move ();

	void leave ();

};
#endif
