#include "audio.h"
#include "chrono.h"
#include <math.h>
#include <fstream>
#include <stdlib.h>
using namespace std;

extern int SAMPLE_RATE;
extern double SECS_PER_BUFFER;

string name ("din"), dinl, dinr;
static const char* colon = ":", *L = "L", * R = "R";

extern string dotdin;

audio_out::audio_out () {

	prefs_name = dotdin + "jack_prefs";
	load_prefs ();
	open ();
}

audio_out::~audio_out () {

	close ();

	float* ptr [] = {samples_buffers, result, ams, fms};
	for (int i = 0; i < 4; ++i) delete[] ptr[i];

	delete[] available;

}

void audio_out::alloc () {

	samples_per_buffer = num_channels * samples_per_channel;
	samples_buffer_size = sizeof (float) * samples_per_buffer;

	if (samples_buffers) delete[] samples_buffers;
	samples_buffers = new float [num_samples_buffers * samples_per_buffer];

	if (available) delete[] available;
	available = new int [num_samples_buffers];

	readi = writei = 0;
	readp = writep = samples_buffers;
	for (int i = 0; i < num_samples_buffers; ++i) available[i] = 0;

	float** ptr [] = {&result, &ams, &fms};
	for (int i = 0; i < 3; ++i) {
		if (ptr[i]) delete[] *ptr[i];
		*ptr[i] = new float [samples_per_channel];
	}

}

void audio_out::set_sample_rate (int s) {

	SAMPLE_RATE = sample_rate = s;
	SECS_PER_BUFFER = samples_per_channel * 1. / SAMPLE_RATE;
	calc_tonic_limits ();
}

int audio_out::open() {

	cout << "<<< connecting to JACK server >>>" << endl;
	jack_status_t status;
	jack_options_t options = JackNullOption;

	client = jack_client_open (name.c_str(), options, &status, 0);
	if (client == 0) {
		cout << "!!! couldnt connect to JACK server. !!!" << endl;
		exit (1);
	} else {
		name = string (jack_get_client_name (client));
		cout << "+++ connected to JACK server as: " << name << " +++" << endl;
		dinl = name + colon + L;
		dinr = name + colon + R;
	}

	sample_rate = jack_get_sample_rate (client);
	jack_set_sample_rate_callback (client, sample_rate_changed, this);

	samples_per_channel = jack_get_buffer_size (client);
	jack_set_buffer_size_callback (client, buffer_size_changed, this);

	out_left = jack_port_register (client, L, JACK_DEFAULT_AUDIO_TYPE, JackPortIsOutput, 0);
	out_right = jack_port_register (client, R, JACK_DEFAULT_AUDIO_TYPE, JackPortIsOutput, 0);
	if ((out_left == 0) || (out_right == 0)) {
		cout << "!!! couldnt create output ports !!!" << endl;
		exit (0);
	} else cout << "+++ opened L and R for output +++" << endl;
	lcons = rcons = 0;

	alloc ();
	set_sample_rate (sample_rate);

	return 0;

}

int audio_out::close () {
	if (client) jack_client_close (client);
	return 0;
}

int audio_out::start () {

	if (client) {

		jack_set_process_callback (client, audio_wanted, this);
		jack_activate (client);

		if (lcons) { // restore connections of L
			for (int i = 0; i < lcons; ++i) jack_connect (client, dinl.c_str(), con_left[i]);
			free (con_left);
		} else if (jack_connect (client, dinl.c_str(), "system:playback_1") != 0) {
			cout << "!!! couldnt connect to playback_1 !!!" << endl;
		}

		if (rcons) { // restore connections of R
			for (int i = 0; i < lcons; ++i) jack_connect (client, dinr.c_str(), con_right[i]);
			free (con_right);
		} else if (jack_connect (client, dinr.c_str(), "system:playback_2") != 0) {
			cout << "!!! couldnt connect to playback_2 !!!" << endl;
		}

		cout << "+++ streaming audio now +++ " << endl;

	}
	return 0;
}

int audio_out::stop () {
	if (client) {
		lcons = jack_port_connected (out_left);
		rcons = jack_port_connected (out_right);
		if (lcons) con_left = jack_port_get_connections (out_left);
		if (rcons) con_right = jack_port_get_connections (out_right);
		jack_deactivate (client);
	}
	return 0;
}

void audio_out::write (float* sbuf, float* lbuf, float* rbuf) {

	for (int i = 0; i < samples_per_buffer; i += 2) {
		*lbuf++ = *sbuf++;
		*rbuf++ = *sbuf++;
	}

}

void audio_out::load_prefs () {

	ifstream file (prefs_name.c_str(), ios::in);
	string ignore;

	if (!file)
		defaults ();
	else {
		string type;
		file >> ignore >> type;
		if (type != "audio_prefs") {
			cout << "bad audio prefrences file, please check: " << prefs_name << ", will use defaults." << eol;
			num_samples_buffers = 2;
		} else {
			cout << "<< loading audio prefs from: " << prefs_name;
			file >> ignore >> num_samples_buffers;
			cout << ", loaded. >>>" << eol;
		}
	}
}

void audio_out::save_prefs () {
	ofstream file (prefs_name.c_str(), ios::out);
	if (file) {
		file << "type: audio_prefs" << eol;
		file << "num_samples_buffers: " << num_samples_buffers << eol;
	} else cout << "couldnt write file: " << prefs_name << eol;
}

void audio_out::defaults () {
	cout << "<<< using default for JACK settings >>>" << endl;
	num_samples_buffers = 2;
}

int buffer_size_changed (jack_nframes_t nframes, void *arg) {return 0;}
int sample_rate_changed (jack_nframes_t nframes, void *arg) {return 0;}

extern audio_out aout;
extern chrono clk;

int audio_wanted (jack_nframes_t nsamples, void *arg) {
	if (aout.available[aout.readi]) {
		sample_t* lbuf = (sample_t *) jack_port_get_buffer (aout.out_left, aout.samples_per_channel); // left buf
		sample_t* rbuf = (sample_t *) jack_port_get_buffer (aout.out_right, aout.samples_per_channel); // right buf
		aout.write (aout.readp, lbuf, rbuf);
		aout.available [aout.readi] = 0;
		if (++aout.readi >= aout.num_samples_buffers) {
			aout.readi = 0;
			aout.readp = aout.samples_buffers;
		} else aout.readp += aout.samples_per_buffer;
		clk += aout.samples_per_buffer;
	}
	return 0;
}
