#include "globals.h"
#include "din.h"
#include "command.h"
#include "console.h"
#include "math.h"
#include "delay.h"
#include "scalelist.h"
#include "font.h"
#include "glyph.h"
#include "chrono.h"
#include "utils.h"
#include "ui_list.h"
#include "curve_editor.h"
#include "bot.h"
#include "tokenizer.h"
#include "tcl_interp.h"
#include "chrono.h"

#define buf cmdlst.buf
#define result cmdlst.result

extern console cons;
extern cmdlist cmdlst;
extern tcl_interp interpreter;

void cmdlist::add (command* cmd) {
	cmds.push_back (cmd);
	ncmds = cmds.size ();
}

extern string SCALE;
extern string TUNING;

bool setv::operator() (tokenizer& tz) {

	string varn; tz >> varn;
	if (varn == "sc" || varn == "scroll") {
		string x, y; tz >> x >> y;
		int ix = d->dinfo.scroll.x, iy = d->dinfo.scroll.y;
		if (x != ".") ix = atoi (x.c_str());
		if (y != ".") iy = atoi (y.c_str());
		d->dinfo.scroll = point<int>(ix, iy);
	} else if (varn == "fmd" || varn == "fm_depth") {
		tz >> d->fm_depth;
		hz2step (d->fm_depth, d->fm_step);
	} else if (varn == "amd" || varn == "am_depth") {
		tz >> d->am_depth;
	} else if (varn == "dmv" || varn == "drone_master_volume") {
		float f; tz >> f;
		d->update_drone_master_volume (f - d->drone_master_volume);
	} else if (varn == "snap_drones" || varn == "sd") {
		tz >> d->snap_drones;
	} else if (varn == "tu" || varn == "tuning") {
		extern load_globals lg;
		string name; tz >> name;
		string fname = name + ".tuning";
		if (lg.load_intervals (fname)) {
			TUNING = name;
			result = "tuning is " + name;
			d->restore_all_notes ();
			return true;
		} else {
			result = "bad tuning";
			return false;
		}
	} else if (varn == "ps1" || varn == "prompt") {
		tz >> cons.ps1;
		cons.calc_draw_params ();
	}	else {
		result = varn + ": not found.";
		return false;
	}
	return true;
}


bool getv::operator() (tokenizer& tz) {
	string varn; tz >> varn;
	static const char* ff = "%.3f";
	if (varn == "sc" || varn == "scroll") {
		sprintf (buf, "x %d y %d", d->dinfo.scroll.x, d->dinfo.scroll.y);
	} else if (varn == "fmd" || varn == "fm_depth") {
		sprintf (buf, ff, d->fm_depth);
	} else if (varn == "amd" || varn == "am_depth") {
		sprintf (buf, ff, d->am_depth);
	} else if (varn == "dmv" || varn == "drone_master_volume") {
		sprintf (buf, "%f", d->drone_master_volume);
	} else if (varn == "sd" || varn == "snap_drones") {
		sprintf (buf, "%d", d->snap_drones);
	} else if (varn == "tu" || varn == "tuning") {
		result = TUNING;
		return true;
	}else if (varn == "scale") {
		result = SCALE;
		return true;
	} else {
		result = varn + ": not found.";
		return false;
	}
	result = buf;
	return true;
}

bool set_delay::operator() (tokenizer& tz) {

	string name;
	float value;

	delay* d[2] = {left, right};
	delay** pd = d;
	int n = -1;

	tz >> name >> value;

	if (name == "a" || name == "all") n = 2;
	else if (name == "l" || name == "left") n = 1;
	else if (name == "r" || name == "right") {
		pd++;
		n = 1;
	} else {
		result = "bad delay name";
		return false;
	}

	for (int i = 0; i < n; ++i, pd++) {
		(*pd)->set (value);
	}

	return true;

}

bool get_delay::operator() (tokenizer& tz) {

	delay* d;
	string s; tz >> s;
	if (s == "left" || s == "l") d = left;
	else if (s == "right" || s == "r") d = right;
	else {
		result = "bad delay name";
		return false;
	}

	float t; d->get (t);
	sprintf (buf, "%0.2f", t);
	result = buf;

	return true;

}

extern din din0;
const char* bpm_com::str [bpm_com::NUM] = {"gl", "gr", "am", "fm", "os"};
beat2value* bpm_com::bv [] = { &din0.gatl, &din0.gatr, &din0.am, &din0.fm, &din0.octave_shift };

int bpm_com::getid (const string& name) {

	for (int i = 0; i < NUM; ++i) {
		if (name == str[i]) {
			return i;
		}
	}
	return -1;
}

bool ls_bpm::operator() (tokenizer& tz) {
	static const char spc = ' ';
	for (int i = 0;i < NUM; ++i) {
		result = result + str[i] + spc;
	}
	return true;
}

bool set_bpm::operator() (tokenizer& tz) {
	string name;
	float val;
	string rst;
	tz >> name >> val >> rst;
	int i = getid (name);
	if (i == -1) {
		result = "bad bpm name";
		return false;
	} else {
		int reset = 0; if (rst == "reset" || rst == "r") reset = 1;
		bv[i]->set_bpm (val, reset);
		return true;
	}
}

bool get_bpm::operator() (tokenizer& tz) {
	string name; tz >> name;
	int i = getid (name);
	if (i == -1) {
		result = "bad bpm name";
		return false;
	} else {
			sprintf (buf, "%f", bv[i]->get_bpm());
			result = buf;
			return true;
	}

}

bool set_style::operator() (tokenizer& tz) {

	string name, style;
	tz >> name >> style;

	xhandler *xmin = 0, *xmax = 0;
	if (style == "loop" || style == "l") {
		xmin = &_loopmin;
		xmax = &_loopmax;
	} else if (style == "pong" || style == "p") {
		xmin = &_pongmin;
		xmax = &_pongmax;
	} else {
		result = "bad style";
		return false;
	}

	int i = getid (name);

	if (i == -1) {
		result = "bad bpm name";
		return false;
	} else {
			beat2value* bvi = bv[i];
			bvi->xmin = xmin;
			bvi->xmax = xmax;
			bvi->style = style;
			return true;
	}
}

bool get_style::operator() (tokenizer& tz) {
	string name; tz >> name;
	int i = getid (name);
	if (i == -1) {
		result = "bad bpm name";
		return false;
	} else {
			result = bv[i]->style;
			return true;
	}
}

bool gaters::operator() (tokenizer& tz) {
	string target, state; tz >> target >> state;
	if (state == "on") {
		set_state (target, true);
		return true;
	} else if (state == "off") {
		set_state (target, false);
		return true;
	} else {
		result = get_state (target);
		return true;
	}
}

void gaters::set_state (const string& target, const bool& state) {
	if (target == "gl" || target == "gate-l") d->gatl.enabled = state;
	else if (target == "gr" || target == "gate-r") d->gatr.enabled = state;
	else result = "bad gater name";
}

string gaters::get_state (const string& target) {
	bool state = false;
	if (target == "gl" || target == "gate-l") state = d->gatl.enabled;
	else if (target == "gr" || target == "gate-r") state = d->gatr.enabled;
	else return "bad gater name";
	if (state) return "on"; else return "off";
}

bool add_scale::operator() (tokenizer& tz) {
	extern scalelist scalelst;
	scalelst.add (tz);
	return true;
}

bool ls_scale::operator() (tokenizer& tz) {
	extern scalelist scalelst;
	scalelst.ls ();
	return true;
}

bool remove_scale::operator() (tokenizer& tz) {
	string nam; tz >> nam;
	if (nam != "") {
		extern scalelist scalelst;
		scalelst.remove (nam);
		return true;
	}
	result = "bad scale name";
	return false;
}

bool load_scale::operator() (tokenizer& tz) {
	string name; tz >> name;
	if (name != "") {
		extern scalelist scalelst;
		scale r;
		if (scalelst.get (name, r)) {
			d->save_scale ();
			d->scaleinfo.set_scale (name);
			d->load_scale ();
			d->enter ();
			SCALE = name;
		} else {
			result = name + ": not found";
			return false;
		}
	} else {
		result = name + ": bad scale name";
		return false;
	}
	result = "scale is " + name;
	return true;
}

bool ls_notes::operator() (tokenizer& tz) {
	string name; tz >> name;
	extern scalelist scalelst;
	scale r;
	if (scalelst.get (name, r)) {
		result = r.notes;
		return true;
	} else {
		result = name + ": bad scale";
		return false;
	}
}

bool set_kern::operator() (tokenizer& tz) {

	extern font fnt;

	char a, b;
	int k;

	tz >> a >> b >> k;

	if (a == '.') {
		map<char, glyph>& m = fnt.characters;
		for (map<char, glyph>::iterator i = m.begin(), j = m.end(); i != j; ++i) fnt.kern[(*i).first][b] = k * fnt.xsize;
		fnt.modified = 1;
	} else if (b == '.') {
		map<char, glyph>& m = fnt.characters;
		for (map<char, glyph>::iterator i = m.begin(), j = m.end(); i != j; ++i) fnt.kern[a][(*i).first] = k * fnt.xsize;
		fnt.modified = 1;
	} else {
		fnt.kern[a][b] = k * fnt.xsize;
		fnt.modified = 1;
	}

	return true;

}

bool get_kern::operator() (tokenizer& tz) {

	extern font fnt;

	char a, b;
	int k;

	tz >> a >> b >> k;

	if (a == '.') {
		map<char, glyph>& m = fnt.characters;
		for (map<char, glyph>::iterator i = m.begin(), j = m.end(); i != j; ++i) {
			a = (*i).first;
			cons << a << ' ' << b << ' ' << fnt.kern[a][b] / fnt.xsize << eol;
		}
	} else if (b == '.') {
		map<char, glyph>& m = fnt.characters;
		for (map<char, glyph>::iterator i = m.begin(), j = m.end(); i != j; ++i) {
			b = (*i).first;
			cons << a << ' ' << b << ' ' << fnt.kern[a][b] / fnt.xsize << eol;
		}
	} else {
		cons << a << ' ' << b << ' ' << fnt.kern[a][b] / fnt.xsize << eol;
	}

	return true;

}

bool set_font_size::operator() (tokenizer& tz) {

	extern font fnt;

	int xsize, ysize, charspc, headroom;
	tz >> xsize >> ysize >> charspc >> headroom;

	if (xsize < 1) xsize = fnt.xsize;
	if (ysize < 1) ysize = fnt.ysize;
	if (charspc < 1) charspc = fnt.charspc;
	if (headroom < 1) headroom = fnt.headroom;

	fnt.xsize = xsize;
	fnt.ysize = ysize;
	fnt.load (fnt.fname);

	fnt.charspc = charspc;
	fnt.headroom = headroom;
	fnt.wordspc = 2 * fnt.charspc;

	extern console cons;
	cons.calc_draw_params ();

	return true;

}

bool get_font_size::operator() (tokenizer& tz) {
	sprintf (buf, "%d %d %d %d", fnt.xsize, fnt.ysize, fnt.charspc, fnt.headroom);
	result = buf;
	return true;
}

bool load_resonators ::operator() (tokenizer& tz) {
	string fname; tz >> fname;
	fname += ".res";
	if (fname == ".") d->load_resonators (d->dinfo.resonators_fname); else d->load_resonators (fname);
	return true;
}

bool save_resonators ::operator() (tokenizer& tz) {
	string fname; tz >> fname;
	if (fname != ".") d->dinfo.resonators_fname = fname;
	int deldis = 0; tz >> deldis;
	d->save_resonators (deldis);
	return true;
}

bool delete_disabled_resonators ::operator() (tokenizer& tz) {
	cons ("sre . 1");
	cons ("lre .");
	return true;
}

const char* western1 [] = {
	"C", "C#", "D", "D#", "E", "F", "F#", "G", "G#", "A", "A#", "B", "C"
};

const char* western2 [] = {
	"C", "Db", "D", "Eb", "E", "F", "F#", "G", "Ab", "A", "Bb", "B", "C"
};

const char* indian [] = {
	"s", "r", "R", "g", "G", "m", "M", "P", "d", "D", "n", "N", "S"
};

const int NUM_NOTES = 13;

const float frequencies [] = { // from wikipedia - piano key frequencies page.
	261.626, 277.183, 293.665, 311.127, 329.628, 349.228, 369.994, 391.995, 415.305, 440, 466.164, 493.883, 523.251
};

int get_note_num (const string& s) {
	for (int i = 0; i < NUM_NOTES; ++i) {
		if (s == western2[i] || s == western1[i]) return i;
	}
	return -1;
}

bool note_distance::operator() (tokenizer& tz) {

	string note1, note2, verbose;
	tz >> note1 >> note2 >> verbose;
	if (note1.length() && note2.length()) {
		note1[0] = toupper (note1[0]);
		note2[0] = toupper (note2[0]);
	} else return false;

	static const char* dist1 [] = {
		"1", "2b", "2", "3b", "3", "4", "4#", "5", "6b", "6", "7b", "7", "8"
	};
	static const char* dist2 [] = {
		"same", "minor_second", "second", "minor_third", "third", "perfect_fourth", "diminished_fifth", "perfect_fifth", "minor_sixth", "sixth", "minor_seventh", "seventh", "octave"
	};

	int i = get_note_num (note1), j = get_note_num (note2);
	if (i == -1 || j == -1) return false; else {
		if (j <= i) j += 12;
		int ji = j - i;
		if (verbose.length()) {
			sprintf (buf, "%s %s", dist1[ji], dist2[ji]);
			result = buf;
			return true;
		} else {
			result = dist1[ji];
			return true;
		}
	}

}

bool chord::operator() (tokenizer& tz) {

	string root, type;
	tz >> root >> type;

	if (root.length()) root[0] = toupper (root[0]); else return false;
	int nn = get_note_num (root);
	if (nn == -1) {
		result = root + ": bad note";
		return false;
	}else {

		const int ntypes = 20;
		static const char* types[] = {
			"", // major
			"m", // minor
			"maj7", // major 7
			"m7", // minor 7
			"sus4", // suspended 4th
			"7sus4", // dominant 7th, suspended 4th
			"6", // major 6th
			"m6", // minor 6th
			"7", // dominant 7th
			"9", // dominant 9th
			"5", // power chord
			"69", // maj 6th, add 9
			"11", // dominant 11th
			"13", // dominant 13th
			"add9", // major add 9th
			"m9", // minor 9th
			"maj9", // major 9th
			"+", // augmented
			"07", // diminished 7th
			"0" // diminished triad
		};

		static const int steps[][7] = {
			{3, 0, 4, 7}, // num_notes, root, note1, note2....
			{3, 0, 3, 7},
			{4, 0, 4, 7, 11},
			{4, 0, 3, 7, 10},
			{3, 0, 5, 7},
			{4, 0, 5, 7, 10},
			{4, 0, 4, 7, 9},
			{4, 0, 3, 7, 9},
			{4, 0, 4, 7, 10},
			{5, 0, 4, 7, 10, 14},
			{2, 0, 7},
			{5, 0, 4, 7, 9, 14},
			{6, 0, 4, 7, 10, 14, 17},
			{6, 0, 4, 7, 10, 14, 21},
			{4, 0, 4, 7, 14},
			{5, 0, 3, 7, 10, 14},
			{5, 0, 4, 7, 11, 14},
			{3, 0, 4, 8},
			{4, 0, 3, 6, 10},
			{3, 0, 3, 6}
		};

		int t = -1;
		for (int i = 0; i < ntypes; ++i) {
			if (type == types[i]) {
				t = i; break;
			}
		}

		if (t == -1) return false; else {
			const int* ch = &steps[t][0];
			stringstream ss1, ss2;
			for (int i = 0, j = ch[0], k = 1; i < j; ++i, ++k) {
				int p = (nn + ch[k]) % 12;
				ss1 << western1[p] << ' ';
				ss2 << western2[p] << ' ';
			}
			result = ss2.str () + " or " + ss1.str ();
			return true;
		}


	}
}

void key::find_nearest_note (string& note, float& frequency, float& dist) {

	float left = frequencies[0] / 16, right = 2 * left;
	float tonic = d->get_tonic ();
	if (tonic < left) return;

	while (1) {
		if ((left <= tonic) && (right > tonic)) break;
		else {
			left*=2;
			right*=2;
		}
	}

	float oct = left / frequencies[0];
	dist = tonic - left;
	int id = 0;
	float tone = 0;
	for (int i = 0; i < 13; ++i) {
		tone = frequencies[i] * oct;
		float newdist = tone - tonic; if (newdist < 0) newdist = -newdist;
		if (newdist < dist) {
			dist = newdist;
			id = i;
		}
	}

	string nn;
	if (western2[id] == western1[id]) nn = western1[id]; else nn = (string)western2[id] + "/" + (string)western1[id];
	note = nn;
	frequency = frequencies[id] * oct;
	dist = tonic - frequency;
	d->set_key_id (id);

}

bool key::operator() (tokenizer& tz) {

	string note ("invalid");
	float frequency, dist;

	string s; tz >> s;

	if (s == "") {
		// no args, just print key
		find_nearest_note (note, frequency, dist);
		sprintf (buf, "key %s at %0.3f Hz tonic@ %0.3f Hz distance %0.3f Hz", note.c_str(), frequency, d->get_tonic(), dist);
		result = buf;
		return true;
	} else if (s == "value" || s == "v") {
		sprintf (buf, "%0.3f", d->get_tonic ());
		result = buf;
		return true;
	} else if (s == "note" || s == "n") {
		find_nearest_note (note, frequency, dist);
		result = note;
		return true;
	} else {
		float freq = 0;
		int oct = 0;

		float val = atof (s.c_str());
		if (val <= 0) { // a note
			if (s[0] != '.') {
				s[0] = toupper(s[0]);
				for (int i = 0; i < 12; ++i) {
					if ((s == western1[i]) || (s == western2[i])) {
						freq = frequencies[i];
						break;
					}
				}
			} else freq = d->get_tonic ();
		} else { // a value
			freq = val;
		}

		// octave
		tz >> s;
		oct = atoi (s.c_str());

		// check and set
		freq = freq * pow (2, oct);
		bool tonicok = check (freq);
		if (tonicok) {
			d->set_tonic (freq);
			find_nearest_note (note, freq, dist);
			d->notate_ranges ();
			return true;
		}

	}

	return false;

}

bool key::check (float f) {
	const float min_key = frequencies[0] / 8, max_key = 20000;
	if (f < min_key) {
		result = "tonic too low";
		return false;
	} else if (f > max_key) {
		result = "tonic too high";
		return false;
	} else return true;
}

bool notation::operator () (tokenizer& tz) {

	string s; tz >> s;

	if (s == "numeric" || s == "n") d->set_notation ("numeric"); else
	if (s == "w" || s == "west" || s == "western") d->set_notation ("western");
	else result = d->notation;

	return true;

}

bool echo::operator() (tokenizer& tz) {
	into_lines (tz.cur2end().c_str(), cons);
	return true;
}

extern ui_list uis;


multi_curve* get_curve (const string& name) {

	extern din din0;
	extern delay left_delay, right_delay;

	for (int i = 0, j = din0.num_resonators; i < j; ++i) {
		resonator& ri = din0.resonators[i];
		multi_curve* lst [] = {&ri.strength, &ri.left, &ri.right, &ri.wave};
		multi_curve* crv = check_list (lst, 4, name);
		if (crv) return crv;
	}

	multi_curve* lst [] = {
		&din0.fm.crv, &din0.am.crv, &din0.gatl.crv, &din0.gatr.crv,
		&din0.octave_shift.crv, &din0.drone_wave, &left_delay.fbk_crv, &left_delay.vol_crv,
		&right_delay.fbk_crv, &right_delay.vol_crv
	};

	multi_curve* crv = check_list (lst, 10, name);
	return crv;
}


bool curve_value::operator() (tokenizer& tz) {

	string name; tz >> name;

	multi_curve* crv = get_curve (name);

	if (crv) {

		string what;
		int id;
		float sx, sy;
		tz >> what >> id >> sx >> sy;

		if (what == "v") {
			int carry_tangents = 1;
			crv->set_vertex (id, sx, sy, carry_tangents);
			crv->evaluate ();
			return true;
		} else if (what == "lt") {
			crv->set_left_tangent (id, sx, sy);
			crv->evaluate ();
			return true;
		} else if (what == "rt") {
			crv->set_right_tangent (id, sx, sy);
			crv->evaluate ();
			return true;
		} else {
			result = "invalid target: v lt rt supported";
			return false;
		}

	}
	result = "curve " + name + " not found";
	return false;
}

bool curve_name::operator() (tokenizer& tz) {
	curve_editor* ced = uis.crved;
	if (ced && ced->pik()) {
		multi_curve* crv = ced->curveinfo[ced->pik.crv_id].curve;
		string name; tz >> name;
		if (name != "") crv->name = name;
	}
	return true;
}

bool curve__library::operator() (tokenizer& tz) {
	curve_editor* ced = uis.crved;
	if (!ced) {
		result = "not a curve editor";
		return false;
	}
	curve_library* cl = ced->library;
	if (cl) {
		string cmd; tz >> cmd;
		if (cmd == "a" || cmd == "add") ced->add_curve (); else
		if (cmd == "d" || cmd == "delete") cl->del (); else
		if (cmd == "u" || cmd == "update") ced->replace_curve (); else
		if (cmd == "i" || cmd == "insert") ced->insert_curve (); else {
			int i = atoi (cmd.c_str());
			if (i != 0) cl->move (i);
		}
	}	else {
		result = "no curve library";
		return false;
	}
	return true;
}

bool audio__out::operator() (tokenizer& tz) {

	/*extern audio_out aout;

	string param, value, last_value;
	int mod = 0, alloc = 0, ut = 0;

	while (1) {

		tz >> param >> value;
		if (param == "") break;
		int i = atoi (value.c_str());

		if (param == "dsp" || param == "dsp_name") {
			if (value != "") {
				aout.dsp_name = value;
				mod = 1;
			}
		} else if (param == "sr" || param == "sample_rate") {
			if (i > 0) {
				aout.set_sample_rate (i);
				interpreter ("reset_bpms");
				mod = 1;
				ut = 1;
			}
		} else if (param == "nc" || param == "num_channels") {
			aout.num_channels = i;
			mod = 1;
		} else if (param == "nf" || param == "num_fragments") {
			aout.num_fragments = i;
			mod = 1;
		} else if (param == "fs" || param == "fragment_select") {
			aout.fragment_select = i;
			mod = 1;
		} else if (param == "mab" || param == "max_audio_buffers") {
			if (i > 0) {
				aout.max_audio_buffers = i;
				mod = alloc = 1;
			}
		} else if (param == "spc" || param == "samples_per_channel") {
			if (i > 0) {
				aout.samples_per_channel = i;
				mod = alloc = 1;
			}
		} else if (param == "query" || param == "q") {
			sprintf (buf, "dsp %s num_channels %d sample_rate %d max_audio_buffers %d samples_per_channel %d sample_format %d num_fragments %d fragment_select %d", aout.dsp_name.c_str(), aout.num_channels, aout.sample_rate, aout.max_audio_buffers, aout.samples_per_channel, aout.sample_format, aout.num_fragments, aout.fragment_select);
			result =  buf;
			break;
		}

	}

	if (mod) {
		aout.stop ();
		if (alloc) {
			aout.alloc ();
			aout.set_sample_rate (aout.sample_rate);
		}
		aout.start ();
		if (ut) {
			d.update_range_notes ();
			d.update_drone_tone ();
		}
		return true;
	}*/

	return false;

}

extern ui_list uis;
bool set_curve_editor::operator() (tokenizer& tz) {
	string name;
	int screen;
	tz >> name >> screen;
	uis.set_editor (name, screen);
	return true;
}

bool get_curve_editor::operator() (tokenizer& tz) {
	string what;
	tz >> what;
	if (what == "n" || what == "num_screens") {
		sprintf (buf, "%d", ui_list::MAX_EDITORS);
		result = buf;
		return true;
	} else {
		int i = atoi (what.c_str());
		if (i > 0) {
			ui* u = uis.ed[i-1];
			if (u) {
				result = u->name;
				return true;
			} else {
				result = "invalid_editor";
				return false;
			}
		}
		return false;
	}

}

bool midi::operator() (tokenizer& tz) {
	string cmd; tz >> cmd;
	if (cmd == "" || cmd == "list" || cmd == "ls") {
		list_devices ();
		return true;
	} else if (cmd == "n" || cmd == "num") {
		sprintf (buf, "%d", devices[0].getPortCount ());
		result = buf;
		return true;
	} else if (cmd == "open" || cmd == "o") {
		int port;  tz >> port;
		if (port_opened (port) > -1) {
			result = "port already open";
			return false;
		} else {
			int d = find_free_device ();
			if (d == -1) {
				result = "all devices in use. please close a port and try again.";
				return false;
			} else return open_port_on_device (port, d);
		}
	} else if (cmd == "close" || cmd == "c") {
		int port; tz >> port;
		close_port (port);
	}
	return false;
}

int midi::find_free_device () {
	for (int i = 0; i < MAX_DEVICES; ++i) {
		RtMidi& d = devices[i];
		if (!d.connected) return i;
	}
	return -1;
}

int midi::open_port_on_device (int port, int dev) {
	RtMidi& d = devices[dev];
	if (d.openPort (port) > 0) {
		++nports;
		sprintf (buf, "opened port %d on device %d", port, dev);
		result = buf;
		return 1;
	} else {
		int d = port_opened (port);
		if (d > -1) result = "port already open"; else result = "invalid port";
		return 0;
	}
}

int midi::port_opened (int port) {
	for (int i = 0; i < MAX_DEVICES; ++i) {
		RtMidi& d = devices[i];
		if (d.connected && (d.portnum == port)) return i;
	}
	return -1;
}

void midi::close_port (int port) {
	int dev = port_opened (port);
	if (dev > -1) {
		RtMidi& d = devices[dev];
		d.closePort ();
		sprintf (buf, "closed port %d on device %d", port, dev);
		result = buf;
	} else {
		sprintf (buf, "port %d not open", port);
		result = buf;
	}
}

void midi::list_devices () {
	RtMidiIn& device = devices[0];
	int nports = device.getPortCount ();
	string opened;
	stringstream ss;
	for (int i = 0; i < nports; ++i) {
		int d = port_opened (i);
		if (d > -1) opened = "yes"; else opened = "no";
		ss << "name " << device.getPortName(i) + " port " << i << " opened " << opened << eol;
	}
	result = ss.str();
}

extern tcl_interp interpreter;

void midi::handle_input () {

	extern double MIDI_BPM;
	extern chrono clk;

	static float start_time = 0;
	static int num_clocks = 0;

	vector <unsigned char> msg;

	for (int i = 0, j = 0; (i < MAX_DEVICES) && (j < nports); ++i) {
		RtMidiIn& dev = devices[i];
		if (dev.connected) {
			++j;
			dev.getMessage (&msg);
			int n = msg.size ();
			if (n) {
				unsigned char byte1 = msg[0];
				if (byte1 == 0xf8) { // midi clock
					float elapsed_time = clk.secs - start_time;
					++num_clocks;
					if (elapsed_time >= 1) {
						float f_num_clocks = 1. / elapsed_time * num_clocks;
						double midi_bpm = f_num_clocks * 2.5;
						MIDI_BPM = midi_bpm;
						sprintf (buf, "midibpm %f", midi_bpm);
						interpreter (buf);
						num_clocks = 0;
						start_time = clk.secs;
					}
				} else if ((byte1 & 0xf0) == 0xb0) { // control change
					run ("midi_cc", msg);
				}	else if ((byte1 & 0xf0) == 0x90) { // note on
					run ("midi_note_on", msg);
				} else if ((byte1 & 0xf0) == 0x80) { // note off
					run ("midi_note_off", msg);
				} else if (byte1 == 0xfa) { // midi start
					cons ("midi_start");
					num_clocks = 0;
					start_time = clk.secs;
				}
			}
		}
	}
}

void midi::run (const string& c, const vector <unsigned char>& msg) {

	string cmd (c);
	for (int i = 0, j = msg.size(); i < j; ++i) {
			sprintf (buf, " %d", msg[i]);
			cmd += buf;
	}
	interpreter (cmd);
}

bool set_kb_layout::operator() (tokenizer& tz) {
	string name; tz >> name;
	extern string norm_us, norm_uk, shift_us, shift_uk;
	if (name == "us" || name == "usa") {
		layout = name;
		cons.charset [0] = norm_us;
		cons.charset [1] = shift_us;
	} else if (name == "gb" || name == "uk") {
		layout = name;
		cons.charset [0] = norm_uk;
		cons.charset [1] = shift_uk;
	} else {
		result = layout;
	}
	return true;
}

bool set_scope::operator() (tokenizer& tz) {
	string name; tz >> name;
	float r, g, b; r = g = b = 0;
	if (name == "left" || name == "l") {
		tz >> d->osc.lr >> d->osc.lg >> d->osc.lb;
		return true;
	} else if (name == "right" || name == "r") {
		tz >> d->osc.rr >> d->osc.rg >> d->osc.rb;
		return true;
	} else return false;
}

bool get_scope::operator() (tokenizer& tz) {
	string name; tz >> name;
	if (name == "left" || name == "l") {
		sprintf (buf, "%f %f %f", d->osc.lr, d->osc.lg, d->osc.lb);
		result = buf;
		return true;
	} else if (name == "right" || name == "r") {
		sprintf (buf, "%f %f %f", d->osc.rr, d->osc.rg, d->osc.rb);
		result = buf;
		return true;
	} else return false;
}

bool set_drone::operator() (tokenizer& tz) {

	int id;
	string param;
	float value;

	tz >> param >> id >> value;

	if (param == "volume" || param == "v") {
		if (id > -1 && id < d.num_drones) {
			drone& did = d.drones[id];
			did.vol = value;
			extern float DELTA_VOLUME;
			did.dv = did.vol / DELTA_VOLUME;
			did.player.set (did.step, d.drone_master_volume * did.vol, d.drone_master_volume * did.vol);
			return true;
		} else {
			result = "bad drone id";
			return false;
		}
	} else {
		result = "bad param";
		return false;
	}

}

bool get_drone::operator() (tokenizer& tz) {
	string param;
	tz >> param;
	if (param == "n" || param == "num_drones") {
		sprintf (buf, "%d", d.num_drones);
		result = buf;
		return true;
	} else if (param == "volume" || param == "v") {
		int i; tz >> i;
		if (i > -1 && i < d.num_drones) {
			sprintf (buf, "%f", d.drones[i].vol);
			result = buf;
			return true;
		} else {
			result = "bad drone id";
			return false;
		}
	} else {
		result = "bad param";
		return false;
	}
}

bool set_text_color::operator() (tokenizer& tz) {
	tz >> cons.dec.r >> cons.dec.g >> cons.dec.b;
	return true;
}

bool load_tuning::operator() (tokenizer& tz) {
		extern load_globals lg;
		string name; tz >> name;
		string fname = name + ".tuning";
		if (lg.load_intervals (fname)) {
			result = "tuning: " + name;
			d->restore_all_notes ();
			return true;
		} else {
			result = "bad tuning file: " + fname + " or invalid tuning.";
			return false;
		}
}

bool paste_gaters::operator() (tokenizer& tz) {

	extern din din0;
	multi_curve* crv [] = {&din0.gatl.crv, &din0.gatr.crv};
	solver* slv [] = {&din0.gatl.sol, &din0.gatr.sol};
	for (int i = 0; i < 2; ++i) {
		multi_curve* c = crv[i];
		solver* s = slv[i];
		string name; float r, g, b;
		c->get_name_and_color (name, r, g, b);
		*c = curve_editor::copy;
		c->set_name_and_color (name, r, g, b);
		s->update ();
	}
	return true;
}