#include "osc.h"
#include "din.h"
#include "console.h"
#include "command.h"
#include "tokenizer.h"
#include "tcl_interp.h"
#include <sys/time.h>

extern cmdlist cmdlst;

bool oid::operator() (tokenizer& tz) {
	tz >> name;
	cons << console::yellow << name << " has logged into din." << eol;
	cmdlst.result = "hello, " + name;
	return true;
}

bool oserver::operator() (tokenizer& tz) {

	string port; tz >> port;

	int pt = atoi (port.c_str());

	const int MIN_VALID_PORT = 1024;
	if (pt < MIN_VALID_PORT) {
		cons << console::red << "bad port number: " << pt << ", minimum valid port = " << MIN_VALID_PORT << eol;
		return false;
	}

	float msecs; tz >> msecs;
	if (msecs <= 0) msecs = 1;

	if (server) {
		cons << console::yellow << "closing osc server" << eol;
		lo_server_free (server);
	}

	server = lo_server_new (port.c_str(), 0);
	lo_server_add_method (server, 0, 0, command_handler, 0);

	lo_fd = lo_server_get_socket_fd (server);
	if (lo_fd) {
		cons << console::green << "server started at " << port << ", command timeout is " << msecs << " msecs." << eol;
		tv.tv_sec = 0;
		tv.tv_usec = (__suseconds_t ) (msecs * 1000);
		enabled = true;
		msg = true;
		return true;
	} else cout << "could not create server" << endl;

	return false;

}

void oserver::handle_requests () {
	if (lo_fd > 0) {
		FD_ZERO (&rfds);
		FD_SET(lo_fd, &rfds);
		int retval = select(lo_fd + 1, &rfds, 0, 0, &tv);
		if (retval == -1) return;
		lo_server_recv_noblock(server, 0);
	}
}

void oserver::toggle () {
	enabled = !enabled;
}

oserver::~oserver () {

	if (server) lo_server_free (server);

}

int command_handler (const char *path, const char *types, lo_arg **argv, int argc, void *data, void *user_data) {

	extern oserver srv;
	if (!srv.enabled) return 0;

	tokenizer tz (path);
	extern tcl_interp interpreter;
	interpreter (path);

	return 1;
}

bool oconnect::operator() (tokenizer& tz) {

	string ip, port;
	tz >> ip >> port;

	client = lo_address_new (ip.c_str(), port.c_str());
	if (!client) {
		cons << console::red << "failed to connect: " << ip << " at " << port << eol;
		return false;
	} else cons << console::green << "connected to: " << ip << " at " << port << eol;
	return true;
}

bool osend::operator() (tokenizer& tz) {

	string cmd; tz >> cmd;

	if (client) {
		string arg;
		while (1) {
			tz >> arg;
			if (arg == "") break;
			cmd = cmd + ' ' + arg;
		}
		lo_send (client, cmd.c_str(), "s", cmd.c_str());
		return true;

	} else {
		cons << console::red << "connect to osc server first" << eol;
		return false;
	}
}

bool otransmit::operator() (tokenizer& tz) {
	return true;
}

bool mouse_sender::operator() () {
	if (oo.client && ((d->prev_mousex != d->win_mousex) || (d->prev_mousey != d->win_mousey))) {
		lo_send (oo.client, oo.path.c_str(), oo.format.c_str(), d->win_mousex, d->win_mousey);
		return true;
	} else return false;
}

bool pitch_sender::operator() () {
	float step = d->step;
	if ((step != prev) && oo.client) {
		extern int SAMPLE_RATE;
		float pitch = step * SAMPLE_RATE;
		prev = step;
		lo_send (oo.client, oo.path.c_str(), oo.format.c_str(), pitch);
		return true;
	} else return false;
}

bool sa_sender::operator()() {
	float sa = d->get_tonic ();
	if ((sa != prev) && oo.client) {
		lo_send (oo.client, oo.path.c_str(), oo.format.c_str(), sa);
		prev = sa;
		return true;
	}
	return false;
}

bool win_sender::operator()() {
	if (oo.client) {
		extern viewport view;
		int left = d->ranges[0].extents.left;
		int right = d->ranges[d->num_ranges - 1].extents.right;
		int bottom = 0, top = view.ymax;
		if ((left != pleft) || (right != pright) || (bottom != pbottom) || (top != ptop)) {
			lo_send (oo.client, oo.path.c_str(), oo.format.c_str(), left, bottom, right, top);
			pleft = left; pbottom = bottom;
			pright = right; ptop = top;
			return true;
		}
	}
	return false;

}

bool volume_sender::operator()() {
	if (oo.client) {
		extern int LAST_VOLUME;
		float volume = d->dv * 1./ LAST_VOLUME;
		if (volume < 0) volume = 0;
		if (volume != prev) lo_send (oo.client, oo.path.c_str(), oo.format.c_str(), volume);
		prev = volume;
		return true;
	} else return false;

}

void osc () {

	extern oserver srv;
	srv.handle_requests ();


}


