#include "solver.h"
#include "multi_curve.h"
#include <cmath>
using namespace std;

void solver::init () {
  mcrv = 0;
  icurv = ncurvs = 0;
  iseg = 0;
  firstx = firsty = 0;
  lastx = lasty = 0;
  startx = starty = 0;
  endx = endy = 0;
  result = 0;

}

solver::solver () {
	init ();
}

solver::solver (multi_curve* c) {
	init ();
  operator() (c);
}

void solver::operator() (multi_curve* c) {
  mcrv = c;
	update ();
}

void solver::update () {

	// curve valid?
	vector<curve>& curv = mcrv->curv;
	ncurvs = curv.size ();
	last_curv = ncurvs - 1;
	if (icurv > last_curv) icurv = 0;

	// segment valid?
	vector<crvpt>& vpts = curv[icurv].vpts;
	int nsegs = vpts.size() - 1;
	if (iseg >= nsegs) iseg = 0;

	setseg (icurv, iseg);

	// first point
	vector<crvpt>& fpts = curv[0].vpts;
	firstx = fpts[0].x;
	firsty = fpts[0].y;

	// last point
	vector<crvpt>& lpts = curv[last_curv].vpts;
	int last_point = lpts.size () - 1;
	lastx = lpts[last_point].x;
	lasty = lpts[last_point].y;

}

float solver::operator() (float x) { // solve y for x

	if (x < startx) {
		if (!searchleft (x)) {
			result = firsty;
		}
	} else if (x > endx) {
		if (!searchright (x)) {
			result = lasty;
		}
	} else result = ycomp + m * x;
	return result;
}

// solves y for x with modulation of x
void solver::operator() (float& x, float& dx, int q, float* mod, float* y, xhandler& xmin, xhandler& xmax, int dp) {
	for (int p = 0; p < q; p += dp) {
		x += (mod[p] + dx);
		if (x < startx) {
			if (!searchleft (x)) xmin (*this, x, dx);
		} else if (x > endx) {
			if (!searchright (x)) xmax (*this, x, dx);
		}
		y[p] = ycomp + m * x;
	}
}

// solves y for x
void solver::operator() (float& x, float& dx, int q, float* y, xhandler& xmin, xhandler& xmax, int dp) {
	for (int p = 0; p < q; p += dp) {
		x += dx;
		if (x < startx) {
			if (!searchleft (x)) xmin (*this, x, dx);
		} else if (x > endx) {
			if (!searchright (x)) xmax (*this, x, dx);
		}
		y[p] = ycomp + m * x;
	}
}

// solves array of x. stores solution ie y in the same array at corresponding location
void solver::operator() (float* ax, int q, int dp, xhandler& xmin, xhandler& xmax) {

	float x, dx = 0;
	for (int p = 0; p < q; ++p, ax += dp) {
		x = *ax;
		if (x < startx) {
			if (!searchleft (x)) {
				xmin (*this, x, dx);
			}
		} else if (x > endx) {
			if (!searchright (x)) {
				xmax (*this, x, dx);
			}
		}
		*ax = ycomp + m * x;
	}

}

atmin _atmin;
atmax _atmax;
tomin _tomin;
loopmin _loopmin;
loopmax _loopmax;
pongmin _pongmin;
pongmax _pongmax;

inline void atmin::operator() (solver& s, float& x, float& dx) {
	x = s.firstx;
	s.setseg (0, 0);
}

inline void tomin::operator() (solver& s, float& x, float& dx) {
	x -= s.lastx;
	if (!s.inseg (x, 0, 0)) if (!s.searchright(x)) s.searchleft (x);
}

inline void loopmin::operator() (solver& s, float& x, float& dx) {
	atmin::operator() (s, x, dx);
	if (dx < 0) dx = -dx;
}

inline void loopmax::operator() (solver& s, float& x, float& dx) {
	tomin::operator() (s, x, dx);
	if (dx < 0) dx = -dx;
}

inline void pongmax::operator () (solver& s, float& x, float& dx) {
	x = s.lastx;
	s.setseg (s.last_curv, s.lastseg (s.last_curv));
	if (dx > 0) dx = -dx;
}

inline void atmax::operator () (solver& s, float& x, float& dx) {
	x = s.lastx;
	s.setseg (s.last_curv, s.lastseg (s.last_curv));
	dx = 0;
}


inline void pongmin::operator () (solver& s, float& x, float& dx) {
	x = s.firstx;
	s.setseg (0, 0);
	if (dx < 0) dx = -dx;
}
