#include "tokenizer.h"
#include "command.h"
#include "tcl_interp.h"
#include "random.h"
#include <iostream>
#include <stdint.h>
using namespace std;

extern cmdlist cmdlst;

int tcl_run (ClientData cd, Tcl_Interp* ti, int objc, Tcl_Obj* CONST objv[]) {

	string cmd (Tcl_GetString (objv[0]));
	command* cmdp = cmdlst.cmds[(intptr_t)cd];
	if (cmdp) {
		cmdlst.result = "";
		vector<string> args; for (int i = 1; i < objc; ++i) args.push_back (Tcl_GetString(objv[i]));
		tokenizer tz (args);cmdp->operator() (tz);
		Tcl_SetResult (ti, (char *) cmdlst.result.c_str(), TCL_VOLATILE);
	}
	return 0;
}

void add_commands (Tcl_Interp* ti);

tcl_interp::tcl_interp () {

	Tcl_FindExecutable (0);

	unsafe = Tcl_CreateInterp ();
	safe = Tcl_CreateInterp ();
	if (safe) Tcl_MakeSafe (safe);
	if (!unsafe || !safe) {
		cout << "!!! Could not create Tcl interpreter !!!" << endl;
		exit (1);
	}

	extern double MIDI_BPM;
	extern float TAP_BPM;
	Tcl_Interp* interps [] = {safe, unsafe};
	for (int i = 0; i < 2; ++i) {
		Tcl_Interp* pi = interps[i];
		Tcl_Init (pi);
		add_commands (pi);
		Tcl_LinkVar (pi, "midibpm", (char *) &MIDI_BPM, TCL_LINK_DOUBLE);
		Tcl_LinkVar (pi, "tapbpm", (char *) &TAP_BPM, TCL_LINK_DOUBLE);
	}

	use_unsafe ();

	cout << "+++ created Tcl interpreter +++" << endl;

}

void tcl_interp::setup (Tcl_Interp* pi) {


}

void tcl_interp::use_safe () {
	interp = safe;
	Tcl_ResetResult (interp);
}

void tcl_interp::use_unsafe () {
	interp = unsafe;
	Tcl_ResetResult (interp);
}

const char* tcl_interp::operator () (const string& cmd) {
	Tcl_Obj* script = Tcl_NewStringObj (cmd.c_str(), -1);
	Tcl_IncrRefCount (script);
	Tcl_EvalObjEx (interp, script, 0);
	Tcl_DecrRefCount (script);
	result = Tcl_GetStringResult (interp);
	return result;
}

tcl_interp::~tcl_interp () {

	Tcl_Interp* interps [] = {safe, unsafe};
	for (int i = 0; i < 2; ++i) {
		Tcl_UnlinkVar (interps[i], "midibpm");
		Tcl_UnlinkVar (interps[i], "tapbpm");
		Tcl_DeleteInterp (interps[i]);
	}

}