#include "din.h"
#include "tokenizer.h"

string APP_NAME;
int SCREEN_WIDTH, SCREEN_HEIGHT, SCREEN_DEPTH;

int SAMPLE_RATE;
double SECS_PER_BUFFER;
float MIN_TONIC = 40, MAX_TONIC = 11025;

string TUNING;
string INTERVALS_FILE;
int NUM_INTERVALS;
map <string, float> INTERVALS;
map <string, int> NOTE_POS;
string FIRST_INTERVAL_NAME, LAST_INTERVAL_NAME;

int NUM_VOLUMES, LAST_VOLUME;
float DELTA_VOLUME;

int NUM_MICROTONES;
int MAX_HEIGHT = din::MAX_VOLUMES - 1;
int LEFT, BOTTOM, RIGHT, TOP, HEIGHT;

string SCALE;

void calc_tonic_limits () {
	MIN_TONIC = 40; // hz
	MAX_TONIC = SAMPLE_RATE / 4;
}

void calc_volume_vars (int h) {
	NUM_VOLUMES = h;
	DELTA_VOLUME = 1./NUM_VOLUMES;
	LAST_VOLUME = NUM_VOLUMES - 1;
}

bool load_globals::load_intervals (const string& fname) {

	extern string dotdin;
	ifstream file ((dotdin + fname).c_str(), ios::in);
	if (file) {

		INTERVALS_FILE = fname;

		file >> ignore >> NUM_INTERVALS;
		for (int i = 0, last = NUM_INTERVALS - 1, pos = 0; i < NUM_INTERVALS; ++i, ++pos) {

			string note_name;
			float value;

			file >> note_name >> value;

			if (i == 0) FIRST_INTERVAL_NAME = note_name;
			else if (i == last) LAST_INTERVAL_NAME = note_name;

			INTERVALS [note_name] = value;
			NOTE_POS [note_name] = pos;

		}

		return true;


	} else return false;

}

load_globals::load_globals () {
	extern string dotdin;
	string fname = dotdin + "globals";
	ifstream file (fname.c_str(), ios::in);
	if (file) {
		cout << "<<< loading globals from: " << fname << " >>>" << endl;
		file >> ignore >> APP_NAME;
		file >> ignore >> SCREEN_WIDTH >> SCREEN_HEIGHT >> SCREEN_DEPTH;
		load_last_window ();
		file >> ignore >> INTERVALS_FILE;
		file >> ignore >> SCALE;
		file >> ignore >> LEFT >> BOTTOM;
		file >> ignore >> HEIGHT;
		file >> ignore >> NUM_MICROTONES;
		cout << "<<< globals loaded from: " << fname << " >>>" << endl;
	} else {
		cout << "bad globals file, will use defaults " << endl;
		APP_NAME = "din";
		SCREEN_WIDTH = 640; SCREEN_HEIGHT = 480;
		INTERVALS_FILE = "pytha.tuning";
		SCALE = "major";
		LEFT = 0; BOTTOM = 150; HEIGHT = 300;
		NUM_MICROTONES = 150;
	}

	load_intervals (INTERVALS_FILE);
	TOP = BOTTOM + HEIGHT;
	calc_volume_vars (min (HEIGHT, MAX_HEIGHT));

}

void load_globals::load_last_window () {
	extern string dotdin;
	string fwindow = dotdin + "window";
	ifstream file (fwindow.c_str(), ios::in);
	if (file) {
		cout << "<<< loading last window from: " << fwindow;
		file >> ignore >> SCREEN_WIDTH;
		file >> ignore >> SCREEN_HEIGHT;
		cout << ", done. >>>" << endl;
	}
}




